@@ -70,26 +70,20 @@ class AMQPParams(object):
                  amqp_vhost=None,
                  ssl_enabled=None,
                  ssl_cert_path=None,
+                 ssl_cert_data=None,
                  socket_timeout=3,
                  heartbeat_interval=None):
         super(AMQPParams, self).__init__()
         username = amqp_user or broker_config.broker_username
         password = amqp_pass or broker_config.broker_password
         heartbeat = heartbeat_interval or broker_config.broker_heartbeat
+        ssl_enabled = ssl_enabled or broker_config.broker_ssl_enabled
+        ssl_cert_path = ssl_cert_path or broker_config.broker_cert_path
         credentials = pika.credentials.PlainCredentials(
             username=username,
             password=password,
         )
 
-        broker_ssl_options = {}
-        if ssl_enabled:
-            broker_ssl_options = {
-                'ca_certs': ssl_cert_path,
-                'cert_reqs': ssl.CERT_REQUIRED,
-            }
-        if not broker_ssl_options:
-            broker_ssl_options = broker_config.broker_ssl_options
-            ssl_enabled = ssl_enabled or broker_config.broker_ssl_enabled
         self.raw_host = amqp_host or broker_config.broker_hostname
         self._amqp_params = {
             'port': amqp_port or broker_config.broker_port,
@@ -99,13 +93,30 @@ class AMQPParams(object):
             'socket_timeout': socket_timeout
         }
         if OLD_PIKA:
+            if ssl_cert_data:
+                raise RuntimeError(
+                    'Passing in cert content is incompatible with old pika')
             self._amqp_params['ssl'] = ssl_enabled
-            self._amqp_params['ssl_options'] = broker_ssl_options
+            if ssl_enabled:
+                self._amqp_params['ssl_options'] = {
+                    'cert_reqs': ssl.CERT_REQUIRED,
+                    'ca_certs': ssl_cert_path,
+                }
+            else:
+                self._amqp_params['ssl_options'] = {}
         else:
             if ssl_enabled:
-                self._amqp_params['ssl_options'] = pika.SSLOptions(
-                    ssl.create_default_context(
-                        cafile=broker_ssl_options['ca_certs']))
+                if ssl_cert_data:
+                    ssl_context = ssl.create_default_context(
+                        cadata=ssl_cert_data)
+                elif ssl_cert_path:
+                    ssl_context = ssl.create_default_context(
+                        cafile=ssl_cert_path)
+                else:
+                    raise RuntimeError(
+                        'When ssl is enabled, ssl_cert_path or ssl_cert_data '
+                        'must be provided')
+                self._amqp_params['ssl_options'] = pika.SSLOptions(ssl_context)
 
     def as_pika_params(self):
         return pika.ConnectionParameters(**self._amqp_params)
@@ -582,6 +593,7 @@ def get_client(amqp_host=None,
                amqp_vhost=None,
                ssl_enabled=None,
                ssl_cert_path=None,
+               ssl_cert_data=None,
                name=None,
                connect_timeout=10,
                cls=AMQPConnection):
@@ -598,7 +610,8 @@ def get_client(amqp_host=None,
         amqp_port,
         amqp_vhost,
         ssl_enabled,
-        ssl_cert_path
+        ssl_cert_path,
+        ssl_cert_data,
     )
 
     return cls(handlers=[], amqp_params=amqp_params, name=name,
@@ -663,7 +676,8 @@ def create_events_publisher(amqp_host=None,
                             amqp_port=None,
                             amqp_vhost=None,
                             ssl_enabled=None,
-                            ssl_cert_path=None):
+                            ssl_cert_path=None,
+                            ssl_cert_data=None):
     thread = threading.current_thread()
 
     amqp_params = AMQPParams(
@@ -673,7 +687,8 @@ def create_events_publisher(amqp_host=None,
         amqp_port,
         amqp_vhost,
         ssl_enabled,
-        ssl_cert_path
+        ssl_cert_path,
+        ssl_cert_data,
     )
 
     try:
