@@ -14,14 +14,15 @@
 #    * limitations under the License.
 
 import numbers
+import threading
 from itertools import chain
+from datetime import datetime
 
 from cloudify import constants, utils
 from cloudify.decorators import workflow
 from cloudify.plugins import lifecycle
-from cloudify.manager import get_rest_client
 from cloudify.workflows.tasks_graph import make_or_get_graph
-from cloudify.utils import add_plugins_to_install, add_plugins_to_uninstall
+from cloudify.workflows.tasks import HandlerResult, TASK_SUCCEEDED
 
 
 @workflow(resumable=True)
@@ -692,144 +693,253 @@ def execute_operation(ctx, operation, *args, **kwargs):
     graph.execute()
 
 
-@workflow
-def update(ctx,
-           update_id,
-           added_instance_ids,
-           added_target_instances_ids,
-           removed_instance_ids,
-           remove_target_instance_ids,
-           modified_entity_ids,
-           extended_instance_ids,
-           extend_target_instance_ids,
-           reduced_instance_ids,
-           reduce_target_instance_ids,
-           skip_install,
-           skip_uninstall,
-           ignore_failure=False,
-           install_first=False,
-           node_instances_to_reinstall=None,
-           central_plugins_to_install=None,
-           central_plugins_to_uninstall=None,
-           update_plugins=True):
-    node_instances_to_reinstall = node_instances_to_reinstall or []
-    instances_by_change = {
-        'added_instances': (added_instance_ids, []),
-        'added_target_instances_ids': (added_target_instances_ids, []),
-        'removed_instances': (removed_instance_ids, []),
-        'remove_target_instance_ids': (remove_target_instance_ids, []),
-        'extended_and_target_instances':
-            (extended_instance_ids + extend_target_instance_ids, []),
-        'reduced_and_target_instances':
-            (reduced_instance_ids + reduce_target_instance_ids, []),
+def _set_node_instance_status(task):
+    workflow_context = task.workflow_context
+    instance_id = task.info['instance_id']
+    new_status = _format_system_task_result(task)
+
+    ni = workflow_context.get_node_instance(instance_id)
+    system_properties = ni.system_properties or {}
+    previous_status = system_properties.get('status')
+    system_properties['previous_status'] = previous_status
+    system_properties['status'] = new_status
+
+    workflow_context.update_node_instance(
+        instance_id,
+        force=True,
+        system_properties=system_properties
+    )
+
+
+def _format_system_task_result(task):
+    result = task.async_result.result
+    if isinstance(result, Exception):
+        result = str(result)
+    return {
+        'ok': task.get_state() == TASK_SUCCEEDED,
+        'timestamp': datetime.utcnow().isoformat(),
+        'task': None if task.is_nop() else task.name,
+        'result': result
     }
-    for instance in ctx.node_instances:
-        instance_holders = [instance_holder
-                            for _, (changed_ids, instance_holder)
-                            in instances_by_change.items()
-                            if instance.id in changed_ids]
-        for instance_holder in instance_holders:
-            instance_holder.append(instance)
 
+
+def check_status_on_success(task):
+    """check_status success handler
+
+    This runs when the check_status operation succeeded. Figure out the
+    node-instance based on the given task.info, and store the result into
+    the node-instance's system-properties.
+    """
+    _set_node_instance_status(task)
+    return HandlerResult.cont()
+
+
+def check_status_on_failure(task):
+    """check_status failure handler
+
+    Similar to check_status_on_success, but runs when the check_status
+    task fails.
+    """
+    _set_node_instance_status(task)
+    return HandlerResult.fail()
+
+
+@make_or_get_graph
+def _make_check_status_graph(
+    ctx,
+    run_by_dependency_order=False,
+    type_names=None,
+    node_ids=None,
+    node_instance_ids=None
+):
+    """Make the graph for the check_status workflow
+
+    This is very similar to execute_operation, but a bit simpler, because
+    we only run a single operation from here.
+    """
     graph = ctx.graph_mode()
-    to_install = set(instances_by_change['added_instances'][1])
-    to_uninstall = set(instances_by_change['removed_instances'][1])
+    tasks = {}
+    filtered_node_instances = _filter_node_instances(
+        ctx=ctx,
+        node_ids=node_ids,
+        node_instance_ids=node_instance_ids,
+        type_names=type_names)
 
-    def _install():
-        def _install_nodes():
-            if skip_install:
-                return
-            # Adding nodes or node instances should be based on modified
-            # instances
-            lifecycle.install_node_instances(
-                graph=graph,
-                node_instances=to_install,
-                related_nodes=set(
-                    instances_by_change['added_target_instances_ids'][1])
-            )
+    if run_by_dependency_order:
+        filtered_node_instances_ids = set(inst.id for inst in
+                                          filtered_node_instances)
+        for instance in ctx.node_instances:
+            if instance.id not in filtered_node_instances_ids:
+                tasks[instance.id] = graph.subgraph(instance.id)
 
-            # This one as well.
-            lifecycle.execute_establish_relationships(
-                graph=graph,
-                node_instances=set(
-                    instances_by_change['extended_and_target_instances'][1]),
-                modified_relationship_ids=modified_entity_ids['relationship']
-            )
+    # registering actual tasks to sequences
+    for instance in filtered_node_instances:
+        task = instance.execute_operation(
+            operation='cloudify.interfaces.validation.check_status'
+        )
+        task.info = {'instance_id': instance.id}
+        task.on_success = check_status_on_success
+        task.on_failure = check_status_on_failure
+        graph.add_task(task)
+        tasks[instance.id] = task
 
-        _install_nodes()
-        _install_plugins_on_agent()
+    # adding tasks dependencies if required
+    if run_by_dependency_order:
+        for instance in ctx.node_instances:
+            for rel in instance.relationships:
+                graph.add_dependency(tasks[instance.id],
+                                     tasks[rel.target_id])
+    return graph
 
-    def _uninstall():
-        def _uninstall_nodes():
-            if skip_uninstall:
-                return
-            lifecycle.execute_unlink_relationships(
-                graph=graph,
-                node_instances=set(
-                    instances_by_change['reduced_and_target_instances'][1]),
-                modified_relationship_ids=modified_entity_ids['relationship']
-            )
 
-            lifecycle.uninstall_node_instances(
-                graph=graph,
-                node_instances=to_uninstall,
-                ignore_failure=ignore_failure,
-                related_nodes=set(
-                    instances_by_change['remove_target_instance_ids'][1])
-            )
+@workflow(resumable=True)
+def check_status(ctx, *args, **kwargs):
+    """Run the check_status operation on all node-instances, and store results
 
-        _uninstall_nodes()
-        _uninstall_plugins_on_agent()
+    This will store the check results into each node-instance
+    system_properties.
+    """
+    # never retry the check_status task by default. If users want to retry
+    # some specific check_status for a given node, then can still override
+    # it, in the interfaces declaration
+    workflows = ctx.internal.bootstrap_context.setdefault('workflows', {})
+    workflows['task_retries'] = 0
+    graph = _make_check_status_graph(
+        ctx,
+        name='check_status',
+        *args,
+        **kwargs)
+    graph.execute()
 
-    def _reinstall():
-        subgraph = set([])
-        for node_instance_id in node_instances_to_reinstall:
-            subgraph |= ctx.get_node_instance(
-                node_instance_id).get_contained_subgraph()
-        subgraph -= to_uninstall
-        intact_nodes = set(ctx.node_instances) - subgraph - to_uninstall
-        for n in subgraph:
-            for r in n._relationship_instances:
-                if r in removed_instance_ids:
-                    n._relationship_instances.pop(r)
-        lifecycle.reinstall_node_instances(graph=graph,
-                                           node_instances=subgraph,
-                                           related_nodes=intact_nodes,
-                                           ignore_failure=ignore_failure)
 
-    def _uninstall_plugins_on_agent():
-        if not update_plugins:
-            return
-        _handle_plugin_after_update(
-            ctx, modified_entity_ids['plugin'], 'remove')
+@make_or_get_graph
+def _make_check_drift_graph(
+    ctx,
+    run_by_dependency_order=False,
+    type_names=None,
+    node_ids=None,
+    node_instance_ids=None
+):
+    """Make the graph for the check_status workflow
 
-    def _install_plugins_on_agent():
-        if not update_plugins:
-            return
-        _handle_plugin_after_update(
-            ctx, modified_entity_ids['plugin'], 'add')
+    This is very similar to execute_operation, but a bit simpler, because
+    we only run a single operation from here.
+    """
+    graph = ctx.graph_mode()
+    tasks = {}
+    filtered_node_instances = _filter_node_instances(
+        ctx=ctx,
+        node_ids=node_ids,
+        node_instance_ids=node_instance_ids,
+        type_names=type_names)
 
-    def _update_central_plugins():
-        if not update_plugins:
-            return
-        sequence = graph.sequence()
-        add_plugins_to_uninstall(ctx, central_plugins_to_uninstall, sequence)
-        add_plugins_to_install(ctx, central_plugins_to_install, sequence)
-        graph.execute()
+    if run_by_dependency_order:
+        filtered_node_instances_ids = set(inst.id for inst in
+                                          filtered_node_instances)
+        for instance in ctx.node_instances:
+            if instance.id not in filtered_node_instances_ids:
+                tasks[instance.id] = graph.subgraph(instance.id)
 
-    if install_first:
-        _install()
-        _uninstall()
-    else:
-        _uninstall()
-        _install()
-    _reinstall()
+    # registering actual tasks to sequences
+    for instance in filtered_node_instances:
+        task = instance.execute_operation(
+            operation='cloudify.interfaces.lifecycle.check_drift'
+        )
+        task.info = {'instance_id': instance.id}
+        task.on_success = check_drift_on_success
+        task.on_failure = check_drift_on_failure
+        graph.add_task(task)
+        tasks[instance.id] = task
+        for rel in instance.relationships:
+            source_task = rel.execute_source_operation(
+                'cloudify.interfaces.relationship_lifecycle.check_drift')
+            target_task = rel.execute_target_operation(
+                'cloudify.interfaces.relationship_lifecycle.check_drift')
+            source_task.on_success = check_drift_on_success
+            source_task.on_failure = check_drift_on_failure
+            graph.add_task(source_task)
+            graph.add_dependency(source_task, task)
+            source_task.info = {
+                'instance_id': instance.id,
+                'relationship_target': rel.target_id,
+            }
 
-    _update_central_plugins()
+            target_task.on_success = check_drift_on_success
+            target_task.on_failure = check_drift_on_failure
+            graph.add_task(target_task)
+            graph.add_dependency(target_task, task)
+            target_task.info = {
+                'instance_id': rel.target_id,
+                'relationship_source': instance.id,
+            }
 
-    # Finalize the commit (i.e. remove relationships or nodes)
-    client = get_rest_client()
-    client.deployment_updates.finalize_commit(update_id)
+    # adding tasks dependencies if required
+    if run_by_dependency_order:
+        for instance in ctx.node_instances:
+            for rel in instance.relationships:
+                graph.add_dependency(tasks[instance.id],
+                                     tasks[rel.target_id])
+    return graph
+
+
+# check_drift might attempt to update the same instance multiple times
+# concurrently, let's just make the update 100% serialized for now
+_instances_update_lock = threading.Lock()
+
+
+def _set_node_instance_drift(task):
+    formatted_result = _format_system_task_result(task)
+    workflow_context = task.workflow_context
+    instance_id = task.info['instance_id']
+    relationship_source = task.info.get('relationship_source')
+    relationship_target = task.info.get('relationship_target')
+    with _instances_update_lock:
+        ni = workflow_context.get_node_instance(instance_id)
+        system_properties = ni.system_properties or {}
+        if relationship_source:
+            targets = system_properties.setdefault(
+                'target_relationships_configuration_drift', {})
+            targets[relationship_source] = formatted_result
+        elif relationship_target:
+            sources = system_properties.setdefault(
+                'source_relationships_configuration_drift', {})
+            sources[relationship_target] = formatted_result
+        else:
+            system_properties['configuration_drift'] = formatted_result
+
+        workflow_context.update_node_instance(
+            instance_id,
+            force=True,
+            system_properties=system_properties
+        )
+
+
+def check_drift_on_success(task):
+    _set_node_instance_drift(task)
+    return HandlerResult.cont()
+
+
+def check_drift_on_failure(task):
+    _set_node_instance_drift(task)
+    return HandlerResult.fail()
+
+
+@workflow(resumable=True)
+def check_drift(ctx, *args, **kwargs):
+    graph = _make_check_drift_graph(
+        ctx,
+        name='check_drift',
+        *args,
+        **kwargs)
+    graph.execute()
+
+
+@workflow
+def update(ctx, *args, **kwargs):
+    from cloudify_system_workflows.deployment_update.workflow import (
+        update_deployment
+    )
+    return update_deployment(ctx, *args, **kwargs)
 
 
 @workflow(resumable=True)
