@@ -15,13 +15,13 @@
 
 
 import time
+import unittest
 from os import path
 
 import mock
 import testtools
 
 from cloudify import exceptions
-from cloudify.plugins import lifecycle
 from cloudify.decorators import operation
 from cloudify.test_utils import workflow_test
 from cloudify.workflows.workflow_context import (Modification,
@@ -533,51 +533,6 @@ class TestSubgraphWorkflowLogic(testtools.TestCase):
         self.assertEqual(4, len(all_invocations))
 
 
-def mock_uninstall(graph, node_instances, ignore_failure, related_nodes=None):
-    raise RuntimeError('Test - ignore_failure is: ' + str(ignore_failure))
-
-
-class TestUpdateIgnoreFailure(testtools.TestCase):
-
-    def __init__(self, *args, **kwargs):
-        super(TestUpdateIgnoreFailure, self).__init__(*args, **kwargs)
-        self.original_uninstall = lifecycle.uninstall_node_instances
-
-    def setUp(self):
-        super(TestUpdateIgnoreFailure, self).setUp()
-        lifecycle.uninstall_node_instances = mock_uninstall
-
-    def tearDown(self):
-        lifecycle.uninstall_node_instances = self.original_uninstall
-        super(TestUpdateIgnoreFailure, self).tearDown()
-
-    @workflow_test(path.join(
-        'resources', 'blueprints', 'test-blueprint-ignore-failure.yaml'))
-    def test_update_ignore_failure_false(self, env):
-        try:
-            env.execute('update', parameters={
-                'ignore_failure': False,
-                'modified_entity_ids': {'relationship': ''}
-            })
-        except RuntimeError as e:
-            self.assertEqual('Test - ignore_failure is: False', str(e))
-        else:
-            fail()
-
-    @workflow_test(path.join(
-        'resources', 'blueprints', 'test-blueprint-ignore-failure.yaml'))
-    def test_update_ignore_failure_true(self, env):
-        try:
-            env.execute('update', parameters={
-                'ignore_failure': True,
-                'modified_entity_ids': {'relationship': ''}
-            })
-        except RuntimeError as e:
-            self.assertEqual('Test - ignore_failure is: True', str(e))
-        else:
-            fail()
-
-
 class TestRelationshipOrderInLifecycleWorkflows(testtools.TestCase):
 
     blueprint_path = path.join('resources', 'blueprints',
@@ -660,6 +615,99 @@ class TestRelationshipOrderInLifecycleWorkflows(testtools.TestCase):
         return self.env.storage.get_node_instances(node_id=node_id)[0]
 
 
+class TestCheckStatus(unittest.TestCase):
+    @workflow_test(path.join(
+        'resources',
+        'blueprints',
+        'test-check-status.yaml'))
+    def test_all_nodes_run(self, cfy_local):
+        self.assertRaises(
+            exceptions.WorkflowFailed,
+            cfy_local.execute, 'check_status'
+        )
+        pass_instances = cfy_local.storage.get_node_instances(
+            node_id='node_passing')
+        fail_instances = cfy_local.storage.get_node_instances(
+            node_id='node_failing')
+        noop_instances = cfy_local.storage.get_node_instances(
+            node_id='node_undefined')
+        assert len(pass_instances) == 1
+        assert len(fail_instances) == 1
+        assert len(noop_instances) == 1
+
+        pass_instance = pass_instances[0]
+        assert pass_instance['system_properties']['status']['ok']
+
+        # the noop instance did run, even though it depends on a failing one
+        noop_instance = noop_instances[0]
+        assert 'status' in noop_instance['system_properties']
+        assert noop_instance['system_properties']['status']['ok']
+
+        fail_instance = fail_instances[0]
+        assert 'status' in fail_instance['system_properties']
+        assert not fail_instance['system_properties']['status']['ok']
+
+    @workflow_test(path.join(
+        'resources',
+        'blueprints',
+        'test-check-status.yaml'))
+    def test_by_dependency_order(self, cfy_local):
+        self.assertRaises(
+            exceptions.WorkflowFailed,
+            cfy_local.execute, 'check_status', parameters={
+                'run_by_dependency_order': True,
+            },
+            allow_custom_parameters=True
+        )
+        fail_instances = cfy_local.storage.get_node_instances(
+            node_id='node_failing')
+        noop_instances = cfy_local.storage.get_node_instances(
+            node_id='node_undefined')
+        assert len(fail_instances) == 1
+        assert len(noop_instances) == 1
+
+        fail_instance = fail_instances[0]
+        assert fail_instance['system_properties']['status']
+        assert not fail_instance['system_properties']['status']['ok']
+
+        # with run_by_dependency_order, the noop instance didnt even run,
+        # because it depends on a failing task
+        noop_instance = noop_instances[0]
+        assert 'status' not in noop_instance['system_properties']
+
+
+class TestCheckDrift(unittest.TestCase):
+    @workflow_test(path.join(
+        'resources',
+        'blueprints',
+        'test-check-drift.yaml'))
+    def test_all_nodes_run(self, cfy_local):
+        self.assertRaises(
+            exceptions.WorkflowFailed,
+            cfy_local.execute, 'check_drift'
+        )
+        pass_instances = cfy_local.storage.get_node_instances(
+            node_id='node_passing')
+        fail_instances = cfy_local.storage.get_node_instances(
+            node_id='node_failing')
+        relation_instances = cfy_local.storage.get_node_instances(
+            node_id='node_related')
+        assert len(pass_instances) == 1
+        assert len(fail_instances) == 1
+        assert len(relation_instances) == 1
+        assert set(pass_instances[0].system_properties.keys()) == {
+            'configuration_drift'
+        }
+        assert set(fail_instances[0].system_properties.keys()) == {
+            'configuration_drift',
+            'target_relationships_configuration_drift'
+        }
+        assert set(relation_instances[0].system_properties.keys()) == {
+            'configuration_drift',
+            'source_relationships_configuration_drift'
+        }
+
+
 class TestRollbackWorkflow(LifecycleBaseTest):
 
     @workflow_test(path.join(
