@@ -336,11 +336,11 @@ class BaseWorkflowTest(object):
         logs = []
 
         # Provide same interface as other log/event functions
-        def mock_stdout_event(event):
+        def mock_stdout_event(event, **kwargs):
             events.append(event)
 
         # Provide same interface as other log/event functions
-        def mock_stdout_log(log):
+        def mock_stdout_log(log, **kwargs):
             logs.append(log)
 
         o_stdout_event = cloudify.logs.stdout_event_out
@@ -536,7 +536,7 @@ class LocalWorkflowTest(BaseWorkflowTest):
 
     def test_workflow_bootstrap_context(self):
         def bootstrap_context(ctx, **_):
-            bootstrap_context = ctx.internal._get_bootstrap_context()
+            bootstrap_context = ctx.internal.bootstrap_context
             self.assertEqual(bootstrap_context, {})
         self._execute_workflow(bootstrap_context)
 
@@ -917,7 +917,8 @@ class FileStorageTest(BaseWorkflowTest, testtools.TestCase):
             pass
         self._execute_workflow(stub_workflow, name=self._testMethodName)
         self.assertTrue(os.path.isdir(
-            os.path.join(self.storage_dir, self._testMethodName)))
+            os.path.join(
+                self.storage_dir, 'blueprints', self._testMethodName)))
 
     def test_persistency(self):
         bootstrap_context = {'stub': 'prop'}
@@ -983,8 +984,11 @@ class FileStorageTest(BaseWorkflowTest, testtools.TestCase):
         def op0(ctx, **_):
             self.assertEquals(
                 ctx.plugin.workdir,
-                os.path.join(self.storage_dir, self._testMethodName,
-                             'workdir', 'plugins', 'p'))
+                os.path.join(
+                    self.storage_dir, 'deployments', self._testMethodName,
+                    'workdir', 'plugins', 'p'
+                )
+            )
 
             work_file = os.path.join(ctx.plugin.workdir, 'work_file')
             self.assertFalse(os.path.exists(work_file))
