@@ -90,7 +90,18 @@ def with_execute_after(f):
 
 
 class WorkflowTask(object):
-    """A base class for workflow tasks"""
+    """A base class for workflow tasks
+
+    A WorkflowTask represents an operation to be executed by the mgmtworker
+    or an agent.
+
+    There's two main kinds of WorkflowTasks - a remote kind, to be sent
+    to a remote mgmtworker/agent and executed there; and a local kind, to
+    be executed in the current process, in a background thread.
+
+    The interface of a WorkflowTask is its apply_async method, which
+    returns a WorkflowTaskResult.
+    """
 
     def __init__(self,
                  workflow_context,
@@ -227,7 +238,6 @@ class WorkflowTask(object):
                 'Cannot deserialize: {0!r}'.format(data))
 
     def dump(self):
-        self.stored = True
         return {
             'id': self.id,
             'name': self.name,
@@ -371,7 +381,12 @@ class WorkflowTask(object):
             # the handler set on the task.
             handler_result = HandlerResult.retry()
             handler_result.retry_after = result.retry_after
-        elif self.on_failure:
+            return handler_result
+
+        if self.is_subgraph:
+            self.error = self.failed_task.error
+
+        if self.on_failure:
             handler_result = self.on_failure(self)
         else:
             handler_result = HandlerResult.retry()
@@ -421,7 +436,10 @@ class WorkflowTask(object):
 
 
 class RemoteWorkflowTask(WorkflowTask):
-    """A WorkflowTask wrapping an AMQP based task"""
+    """A WorkflowTask wrapping an AMQP based task
+
+    This WorkflowTask will be sent via AMQP to a remote mgmtworker/agent.
+    """
     def __init__(self,
                  kwargs,
                  cloudify_context,
@@ -785,6 +803,12 @@ class DryRunLocalWorkflowTask(LocalWorkflowTask):
 
 
 class WorkflowTaskResult(object):
+    """Deferred result of a WorkiflowTask
+
+    This is returned by the WorkflowTask, and will eventually have
+    the actual result set, when the task finishes. Use the on_result method
+    to register callbacks to be called when the task result is set.
+    """
     _NOT_SET = object()
 
     def __init__(self, task):
@@ -997,11 +1021,7 @@ class SendNodeEventTask(_BuiltinTaskBase):
         self.info = event
 
     def remote(self):
-        # only explicitly send the event if the task is not stored - if we do
-        # actually update the operation state, then the server will
-        # automatically add an event
-        if not self.stored:
-            self._send(out_func=logs.manager_event_out)
+        self._send(out_func=logs.manager_event_out)
 
     def local(self):
         self._send(out_func=logs.stdout_event_out)
@@ -1014,7 +1034,11 @@ class SendNodeEventTask(_BuiltinTaskBase):
             event_type='workflow_node_event',
             message=self.kwargs['event'],
             additional_context=self.kwargs['additional_context'],
-            out_func=out_func)
+            out_func=out_func,
+            # if this operation is stored, we don't need to send the log,
+            # because updating the operation state will insert the log
+            # automatically, on the manager side
+            skip_send=self.stored)
 
 
 class SendWorkflowEventTask(_BuiltinTaskBase):
